@@ -1,7 +1,7 @@
 // qsynthSetup.h
 //
 /****************************************************************************
-   Copyright (C) 2003, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,7 +25,6 @@
 #include <qstring.h>
 #include <qstringlist.h>
 #include <qsettings.h>
-#include <qcombobox.h>
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -46,21 +45,20 @@ public:
     // Default destructor.
     ~qsynthSetup();
 
-    // Command line arguments parser.
-    bool parse_args(int argc, char **argv);
-    // Command line usage helper.
-    void print_usage(const char *arg0);
-
     // Settings cache realization.
     void realize();
 
     // Fluidsynth settings accessor.
     fluid_settings_t *fluid_settings();
 
+    // Setup display name.
+    QString sDisplayName;
+
     // Settings variables.
     bool    bMidiIn;
     QString sMidiDriver;
     int     iMidiChannels;
+    QString sAlsaName;
     QString sAudioDriver;
     QString sJackName;
     bool    bJackAutoConnect;
@@ -93,41 +91,17 @@ public:
     QStringList soundfonts;
     QStringList midifiles;
 
-    // Display options...
-    QString sMessagesFont;
-    bool    bQueryClose;
-    bool    bStdoutCapture;
-
-    // Default options...
-    QString sSoundFontDir;
-    bool    bPresetPreview;
+    // Current (translated) preset name.
+    QString sDefPresetName;
 
     // Current (default) preset name.
     QString sDefPreset;
     // Available presets list.
     QStringList presets;
 
-    // Preset management methods.
-    bool loadPreset(fluid_synth_t *pSynth, const QString& sPreset);
-    bool savePreset(fluid_synth_t *pSynth, const QString& sPreset);
-    bool deletePreset(const QString& sPreset);
-
-    // Combo box history persistence helper prototypes.
-    static void add2ComboBoxHistory(QComboBox *pComboBox, const QString& sNewText, int iIndex = 0, int iLimit = 8);
-    void loadComboBoxHistory(QComboBox *pComboBox, int iLimit = 8);
-    void saveComboBoxHistory(QComboBox *pComboBox, int iLimit = 8);
-
-    // Widget geometry persistence helper prototypes.
-    void saveWidgetGeometry(QWidget *pWidget);
-    void loadWidgetGeometry(QWidget *pWidget);
-
 private:
 
-    // Special parsing of '-o' command-line option into fluidsynth settings.
-    bool parse_option(char *optarg);
-
-    // Settings member variables.
-    QSettings m_settings;
+    // Fluidsynth settings member variable.
     fluid_settings_t *m_pFluidSettings;
 };
 
