@@ -2,7 +2,7 @@
 //
 // ui.h extension file, included from the uic-generated form implementation.
 /****************************************************************************
-   Copyright (C) 2003, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2004, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,9 +26,12 @@
 #include <qfiledialog.h>
 #include <qfileinfo.h>
 #include <qfontdialog.h>
+#include <qlistbox.h>
 
 #include "config.h"
 
+#include "qsynthAbout.h"
+
 
 // Our local parameter data struct.
 struct qsynth_settings_data
@@ -151,13 +154,12 @@ static void qsynth_settings_foreach ( void *pvData, char *pszName, int iType )
 }
 
 
-
-
 // Kind of constructor.
 void qsynthSetupForm::init (void)
 {
     // No settings descriptor initially (the caller will set it).
     m_pSetup = NULL;
+    m_pOptions = NULL;
 
     // Initialize dirty control state.
     m_iDirtySetup = 0;
@@ -167,9 +169,13 @@ void qsynthSetupForm::init (void)
     m_pXpmSoundFont  = new QPixmap(QPixmap::fromMimeSource("sfont1.png"));
 
     // Set dialog validators...
+    QRegExp rx("[\\w-]+");
+    DisplayNameLineEdit->setValidator(new QRegExpValidator(rx, DisplayNameLineEdit));
     SampleRateComboBox->setValidator(new QIntValidator(SampleRateComboBox));
     AudioBufSizeComboBox->setValidator(new QIntValidator(AudioBufSizeComboBox));
     AudioBufCountComboBox->setValidator(new QIntValidator(AudioBufCountComboBox));
+    JackNameComboBox->setValidator(new QRegExpValidator(rx, JackNameComboBox));
+    AlsaNameComboBox->setValidator(new QRegExpValidator(rx, AlsaNameComboBox));
 
     // No sorting on soundfont stack list.
     SoundFontListView->setSorting(-1);
@@ -188,23 +194,29 @@ void qsynthSetupForm::destroy (void)
 
 
 // Populate (setup) dialog controls from settings descriptors.
-void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
+void qsynthSetupForm::setup ( qsynthOptions *pOptions, qsynthEngine *pEngine, bool bNew )
 {
-    // Set reference descriptor.
-    m_pSetup = pSetup;
+    // Check this first.
+    if (pOptions == NULL || pEngine == NULL)
+        return;
 
-    // Start clean.
+    // Set reference descriptors.
+    m_pOptions = pOptions;
+    m_pSetup = pEngine->setup();
+
+    // Update caption.
+    setCaption(QSYNTH_TITLE ": " + tr("Setup") + " [" + pEngine->name() + "]");
+
+    // Start clean?
     m_iDirtyCount = 0;
+    if (bNew)
+        m_iDirtyCount++;
     // Avoid nested changes.
     m_iDirtySetup++;
 
-    // Load combobox stories.
-    m_pSetup->loadComboBoxHistory(SoundFontComboBox);
-    SoundFontComboBox->setCurrentText(QString::null);
-
-    // Some defaults first.
-    m_sSoundFontDir = m_pSetup->sSoundFontDir;
-
+    // Display name.
+    DisplayNameLineEdit->setText(m_pSetup->sDisplayName);
+    
     // Load Setttings view...
     qsynth_settings_data data;
     // Set data context.
@@ -214,16 +226,24 @@ void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
     // And start filling it in...
     ::fluid_settings_foreach(m_pSetup->fluid_settings(), &data, qsynth_settings_foreach);
 
-    // Midi Driver combobox options.
+    // Midi Driver combobox options;
+    // check if intended MIDI driver is available.
     data.options.clear();
     ::fluid_settings_foreach_option(m_pSetup->fluid_settings(), "midi.driver", &data, qsynth_settings_foreach_option);
     MidiDriverComboBox->clear();
+    if (data.options.find(m_pSetup->sMidiDriver) == data.options.end())
+        data.options.append(m_pSetup->sMidiDriver);
     MidiDriverComboBox->insertStringList(data.options);
+
     // Audio Driver combobox options.
+    // check if intended Audio driver is available.
     data.options.clear();
     ::fluid_settings_foreach_option(m_pSetup->fluid_settings(), "audio.driver", &data, qsynth_settings_foreach_option);
     AudioDriverComboBox->clear();
+    if (data.options.find(m_pSetup->sAudioDriver) == data.options.end())
+        data.options.append(m_pSetup->sAudioDriver);
     AudioDriverComboBox->insertStringList(data.options);
+
     // Sample Format combobox options.
     data.options.clear();
     ::fluid_settings_foreach_option(m_pSetup->fluid_settings(), "audio.sample-format", &data, qsynth_settings_foreach_option);
@@ -236,6 +256,9 @@ void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
     MidiChannelsSpinBox->setValue(m_pSetup->iMidiChannels);
     MidiDumpCheckBox->setChecked(m_pSetup->bMidiDump);
     VerboseCheckBox->setChecked(m_pSetup->bVerbose);
+    // ALSA client identifier.
+    AlsaNameComboBox->insertItem(m_pSetup->sDisplayName);
+    AlsaNameComboBox->setCurrentText(bNew ? m_pSetup->sDisplayName : m_pSetup->sAlsaName);
 
     // Audio settings...
     AudioDriverComboBox->setCurrentText(m_pSetup->sAudioDriver);
@@ -246,17 +269,25 @@ void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
     AudioChannelsSpinBox->setValue(m_pSetup->iAudioChannels);
     AudioGroupsSpinBox->setValue(m_pSetup->iAudioGroups);
     PolyphonySpinBox->setValue(m_pSetup->iPolyphony);
+    JackMultiCheckBox->setChecked(m_pSetup->bJackMulti);
     JackAutoConnectCheckBox->setChecked(m_pSetup->bJackAutoConnect);
+    // JACK client name...
+    QString sJackName;
+    if (m_pSetup->sDisplayName.find(QSYNTH_TITLE) < 0)
+        sJackName += QSYNTH_TITLE "_";
+    sJackName += m_pSetup->sDisplayName;
+    JackNameComboBox->insertItem(sJackName);
+    JackNameComboBox->setCurrentText(bNew ? sJackName : m_pSetup->sJackName);
 
     // Load the soundfonts view.
     SoundFontListView->clear();
     SoundFontListView->setUpdatesEnabled(false);
     QListViewItem *pItem = NULL;
-    if (pSynth) {
+    if (pEngine->pSynth) {
         // Load soundfont view from actual synth stack...
-        int cSoundFonts = ::fluid_synth_sfcount(pSynth);
+        int cSoundFonts = ::fluid_synth_sfcount(pEngine->pSynth);
         for (int i = cSoundFonts - 1; i >= 0; i--) {
-            fluid_sfont_t *pSoundFont = ::fluid_synth_get_sfont(pSynth, i);
+            fluid_sfont_t *pSoundFont = ::fluid_synth_get_sfont(pEngine->pSynth, i);
             if (pSoundFont) {
                 pItem = new QListViewItem(SoundFontListView, pItem);
                 if (pItem) {
@@ -281,16 +312,6 @@ void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
     SoundFontListView->setUpdatesEnabled(true);
     SoundFontListView->update();
 
-    // Load Display options...
-    QFont font;
-    if (m_pSetup->sMessagesFont.isEmpty() || !font.fromString(m_pSetup->sMessagesFont))
-        font = QFont("Fixed", 8);
-    MessagesFontTextLabel->setFont(font);
-    MessagesFontTextLabel->setText(font.family() + " " + QString::number(font.pointSize()));
-    // Other options finally.
-    QueryCloseCheckBox->setChecked(m_pSetup->bQueryClose);
-    StdoutCaptureCheckBox->setChecked(m_pSetup->bStdoutCapture);
-
     // Done.
     m_iDirtySetup--;
     stabilizeForm();
@@ -300,16 +321,13 @@ void qsynthSetupForm::setup ( qsynthSetup *pSetup, fluid_synth_t *pSynth )
 // Accept settings (OK button slot).
 void qsynthSetupForm::accept (void)
 {
-    // Save options...
-    m_pSetup->sMessagesFont  = MessagesFontTextLabel->font().toString();
-    m_pSetup->bQueryClose    = QueryCloseCheckBox->isChecked();
-    m_pSetup->bStdoutCapture = StdoutCaptureCheckBox->isChecked();
-
     if (m_iDirtyCount > 0) {
         // Save the soundfont view.
         m_pSetup->soundfonts.clear();
         for (QListViewItem *pItem = SoundFontListView->firstChild(); pItem; pItem = pItem->nextSibling())
             m_pSetup->soundfonts.append(pItem->text(1));
+        // Will we have a setup renaming?
+        m_pSetup->sDisplayName     = DisplayNameLineEdit->text();
         // Audio settings...
         m_pSetup->sAudioDriver     = AudioDriverComboBox->currentText();
         m_pSetup->sSampleFormat    = SampleFormatComboBox->currentText();
@@ -319,22 +337,20 @@ void qsynthSetupForm::accept (void)
         m_pSetup->iAudioChannels   = AudioChannelsSpinBox->value();
         m_pSetup->iAudioGroups     = AudioGroupsSpinBox->value();
         m_pSetup->iPolyphony       = PolyphonySpinBox->value();
+        m_pSetup->bJackMulti       = JackMultiCheckBox->isChecked();
         m_pSetup->bJackAutoConnect = JackAutoConnectCheckBox->isChecked();
+        m_pSetup->sJackName        = JackNameComboBox->currentText();
         // Midi settings...
         m_pSetup->bMidiIn          = MidiInCheckBox->isChecked();
         m_pSetup->sMidiDriver      = MidiDriverComboBox->currentText();
         m_pSetup->iMidiChannels    = MidiChannelsSpinBox->value();
         m_pSetup->bMidiDump        = MidiDumpCheckBox->isChecked();
         m_pSetup->bVerbose         = VerboseCheckBox->isChecked();
+        m_pSetup->sAlsaName        = AlsaNameComboBox->currentText();
         // Reset dirty flag.
         m_iDirtyCount = 0;
     }
 
-    // Save combobox stories.
-    m_pSetup->saveComboBoxHistory(SoundFontComboBox);
-    // And the defaults too.
-    m_pSetup->sSoundFontDir = m_sSoundFontDir;
-
     // Just go with dialog acceptance.
     QDialog::accept();
 }
@@ -366,8 +382,33 @@ void qsynthSetupForm::reject (void)
 }
 
 
+// Dirty up engine display name.
+void qsynthSetupForm::nameChanged ( const QString& )
+{
+    if (m_iDirtySetup > 0)
+        return;
+
+    QListBox *pListBox;
+    const QString& sDisplayName = DisplayNameLineEdit->text();
+
+    pListBox = AlsaNameComboBox->listBox();
+    if (pListBox->findItem(sDisplayName, Qt::ExactMatch) == NULL) {
+        pListBox->removeItem(pListBox->count() - 1);
+        pListBox->insertItem(DisplayNameLineEdit->text());
+    }
+
+    pListBox = JackNameComboBox->listBox();
+    if (pListBox->findItem(sDisplayName, Qt::ExactMatch) == NULL) {
+        pListBox->removeItem(pListBox->count() - 1);
+        pListBox->insertItem(DisplayNameLineEdit->text());
+    }
+
+    settingsChanged();
+}
+
+
 // Dirty up settings.
-void qsynthSetupForm::settingsChanged()
+void qsynthSetupForm::settingsChanged (void)
 {
     if (m_iDirtySetup > 0)
         return;
@@ -378,7 +419,7 @@ void qsynthSetupForm::settingsChanged()
 
 
 // Stabilize current form state.
-void qsynthSetupForm::stabilizeForm()
+void qsynthSetupForm::stabilizeForm (void)
 {
     bool bEnabled = MidiInCheckBox->isChecked();
     MidiDriverTextLabel->setEnabled(bEnabled);
@@ -388,12 +429,17 @@ void qsynthSetupForm::stabilizeForm()
     MidiDumpCheckBox->setEnabled(bEnabled);
     VerboseCheckBox->setEnabled(bEnabled);
 
-    JackAutoConnectCheckBox->setEnabled(AudioDriverComboBox->currentText() == "jack");
+    bool bAlsaEnabled = (bEnabled && MidiDriverComboBox->currentText() == "alsa_seq");
+    AlsaNameTextLabel->setEnabled(bAlsaEnabled);
+    AlsaNameComboBox->setEnabled(bAlsaEnabled);
+    
+    bool bJackEnabled = (AudioDriverComboBox->currentText() == "jack");
+    JackMultiCheckBox->setEnabled(bJackEnabled);
+    JackAutoConnectCheckBox->setEnabled(bJackEnabled);
+    JackNameTextLabel->setEnabled(bJackEnabled);
+    JackNameComboBox->setEnabled(bJackEnabled);
 
-    const QString& sSoundFont = SoundFontComboBox->currentText();
-    SoundFontComboBox->setEnabled(true);
-    SoundFontBrowsePushButton->setEnabled(true);
-    SoundFontAddPushButton->setEnabled(!sSoundFont.isEmpty() && QFileInfo(sSoundFont).exists());
+    SoundFontOpenPushButton->setEnabled(true);
     QListViewItem *pSelectedItem = SoundFontListView->selectedItem();
     if (pSelectedItem) {
         SoundFontRemovePushButton->setEnabled(true);
@@ -405,7 +451,16 @@ void qsynthSetupForm::stabilizeForm()
         SoundFontMoveDownPushButton->setEnabled(false);
     }
 
-    OkPushButton->setEnabled(m_iDirtyCount > 0);
+    bEnabled = (m_iDirtyCount > 0);
+    if (bEnabled && m_pSetup) {
+        const QString& sDisplayName = DisplayNameLineEdit->text();
+        if (sDisplayName != m_pSetup->sDisplayName) {
+            bEnabled = (m_pOptions->engines.find(sDisplayName) == m_pOptions->engines.end());
+            if (bEnabled)
+                bEnabled = (sDisplayName != (m_pOptions->defaultSetup())->sDisplayName);
+        }
+    }
+    OkPushButton->setEnabled(bEnabled);
 }
 
 
@@ -417,12 +472,8 @@ void qsynthSetupForm::contextMenu( QListViewItem *pItem, const QPoint& pos, int
     // Build the soundfont context menu...
     QPopupMenu* pContextMenu = new QPopupMenu(this);
 
-    const QString& sSoundFont = SoundFontComboBox->currentText();
-    iItemID = pContextMenu->insertItem(tr("Browse..."), this, SLOT(browseSoundFont()));
-    iItemID = pContextMenu->insertItem(tr("Add"), this, SLOT(addSoundFont()));
-    pContextMenu->setItemEnabled(iItemID, !sSoundFont.isEmpty() && QFileInfo(sSoundFont).exists());
+    iItemID = pContextMenu->insertItem(tr("Open..."), this, SLOT(openSoundFont()));
     pContextMenu->insertSeparator();
-
     bool bEnabled = (pItem != NULL);
     iItemID = pContextMenu->insertItem(tr("Remove"), this, SLOT(removeSoundFont()));
     pContextMenu->setItemEnabled(iItemID, bEnabled);
@@ -433,11 +484,13 @@ void qsynthSetupForm::contextMenu( QListViewItem *pItem, const QPoint& pos, int
     pContextMenu->setItemEnabled(iItemID, (bEnabled && pItem->nextSibling() != NULL));
 
     pContextMenu->exec(pos);
+    
+    delete pContextMenu;
 }
 
 
 // Refresh the soundfont view ids.
-void qsynthSetupForm::refreshSoundFonts()
+void qsynthSetupForm::refreshSoundFonts (void)
 {
     SoundFontListView->setUpdatesEnabled(false);
     int i = 0;
@@ -449,71 +502,66 @@ void qsynthSetupForm::refreshSoundFonts()
 
 
 // Browse for a soundfont file on the filesystem.
-void qsynthSetupForm::browseSoundFont()
+void qsynthSetupForm::openSoundFont()
 {
-    QString sSoundFont = QFileDialog::getOpenFileName(
-            m_sSoundFontDir,                            // Start here.
-            tr("Soundfont files") + " (*.sf2 *.SF2)",   // Filter (XML files)
+    QStringList soundfonts = QFileDialog::getOpenFileNames(
+            tr("Soundfont files") + " (*.sf2 *.SF2)",   // Filter (SF2 files)
+            m_pOptions->sSoundFontDir,                  // Start here.
             this, 0,                                    // Parent and name (none)
-            tr("Soundfont file")                        // Caption.
+            tr("Soundfont files")                       // Caption.
     );
     
-    if (!sSoundFont.isEmpty())
-        SoundFontComboBox->setCurrentText(sSoundFont);
-}
-
-// Load current sound font filename...
-void qsynthSetupForm::addSoundFont()
-{
-    const QString& sSoundFont = SoundFontComboBox->currentText();
-    if (sSoundFont.isEmpty())
-        return;
-
-    // Check if not already there...
-    if (SoundFontListView->findItem(sSoundFont, 1) &&
-        QMessageBox::warning(this, tr("Warning"),
-            tr("Soundfont file already on list") + ":\n\n" +
-            "\"" + sSoundFont + "\"\n\n" +
-            tr("Add anyway?"),
-            tr("OK"), tr("Cancel")) > 0) {
-        return;
-    }
-
-    // Ready to add this soundfont to list.
     QListViewItem *pItem = NULL;
-    if (::fluid_is_soundfont((char *) sSoundFont.latin1())) {
-        pItem = SoundFontListView->selectedItem();
-        QListViewItem *pItem = SoundFontListView->selectedItem();
-        if (pItem)
-            pItem->setSelected(false);
-        else
-            pItem = SoundFontListView->lastItem();
-        pItem = new QListViewItem(SoundFontListView, pItem);
-        if (pItem) {
-            pItem->setPixmap(0, *m_pXpmSoundFont);
-            pItem->setText(1, sSoundFont);
-            pItem->setSelected(true);
-            SoundFontListView->setCurrentItem(pItem);
-            qsynthSetup::add2ComboBoxHistory(SoundFontComboBox, sSoundFont);
-            SoundFontComboBox->setCurrentText(QString::null);
-            m_sSoundFontDir = QFileInfo(sSoundFont).dirPath(true);
-            m_iDirtyCount++;
+    // For avery selected soundfont to load...
+    for (QStringList::Iterator iter = soundfonts.begin(); iter != soundfonts.end(); iter++) {
+        const QString& sSoundFont = *iter;
+        char *pszFilename = (char *) sSoundFont.latin1();
+        // Is it a soundfont file...
+        if (::fluid_is_soundfont(pszFilename)) {
+            // Check if not already there...
+            if (SoundFontListView->findItem(sSoundFont, 1) &&
+                QMessageBox::warning(this, tr("Warning"),
+                    tr("Soundfont file already on list") + ":\n\n" +
+                    "\"" + sSoundFont + "\"\n\n" +
+                    tr("Add anyway?"),
+                    tr("OK"), tr("Cancel")) > 0) {
+                continue;
+            }
+            // Start inserting in the current selected or last item...
+            if (pItem == NULL) {
+                pItem = SoundFontListView->selectedItem();
+                if (pItem)
+                    pItem->setSelected(false);
+                else
+                    pItem = SoundFontListView->lastItem();
+            }
+            // New item on the block :-)
+            pItem = new QListViewItem(SoundFontListView, pItem);
+            if (pItem) {
+                pItem->setPixmap(0, *m_pXpmSoundFont);
+                pItem->setText(1, sSoundFont);
+                pItem->setSelected(true);
+                SoundFontListView->setCurrentItem(pItem);
+                m_pOptions->sSoundFontDir = QFileInfo(sSoundFont).dirPath(true);
+                m_iDirtyCount++;
+            }
+        }
+        else {
+            QMessageBox::critical(this, tr("Error"),
+                tr("Failed to add soundfont file") + ":\n\n" +
+                "\"" + sSoundFont + "\"\n\n" +
+                tr("Please, check for a valid soundfont file."),
+                tr("Cancel"));
         }
-    } else {
-        QMessageBox::critical(this, tr("Error"),
-            tr("Failed to add soundfont file") + ":\n\n" +
-            "\"" + sSoundFont + "\"\n\n" +
-            tr("Please, check for a valid soundfont file."),
-            tr("Cancel"));
     }
-    
+
     refreshSoundFonts();
     stabilizeForm();
 }
 
 
 // Remove current selected soundfont.
-void qsynthSetupForm::removeSoundFont()
+void qsynthSetupForm::removeSoundFont (void)
 {
     QListViewItem *pItem = SoundFontListView->selectedItem();
     if (pItem) {
@@ -527,7 +575,7 @@ void qsynthSetupForm::removeSoundFont()
 
 
 // Move current selected soundfont one position up.
-void qsynthSetupForm::moveUpSoundFont()
+void qsynthSetupForm::moveUpSoundFont (void)
 {
     QListViewItem *pItem = SoundFontListView->selectedItem();
     if (pItem) {
@@ -553,7 +601,7 @@ void qsynthSetupForm::moveUpSoundFont()
 
 
 // Move current selected soundfont one position down.
-void qsynthSetupForm::moveDownSoundFont()
+void qsynthSetupForm::moveDownSoundFont (void)
 {
     QListViewItem *pItem = SoundFontListView->selectedItem();
     if (pItem) {
@@ -572,18 +620,5 @@ void qsynthSetupForm::moveDownSoundFont()
 }
 
 
-// The messages font selection dialog.
-void qsynthSetupForm::chooseMessagesFont()
-{
-    bool  bOk  = false;
-    QFont font = QFontDialog::getFont(&bOk, MessagesFontTextLabel->font(), this);
-    if (bOk) {
-        MessagesFontTextLabel->setFont(font);
-        MessagesFontTextLabel->setText(font.family() + " " + QString::number(font.pointSize()));
-        settingsChanged();
-    }
-}
-
-
 // end of qsynthSetupForm.ui.h
 
